<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\interfaces;

use apexl\Io\modules\user\entities\userEntity;
use apexl\Io\modules\user\enums\PermissionAction;

interface ChecksPermissions
{
    public function checkPermission(PermissionAction $action, userEntity $user, ?string $tableAlias): bool|callable;
}