<?php

declare(strict_types=1);

namespace Apexl\Tests\User\Helpers;

use apexl\Io\modules\user\interfaces\PermissionInterface;

enum TestPermissionsEnum implements PermissionInterface
{
    case HAS_PERMISSION;
    case DOES_NOT_HAVE_PERMISSION;
}


