<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\interfaces;


use Exception;
use Firebase\JWT\Key;

interface JwtKeyProviderInterface
{
    /**
     * @throws Exception
     */
    public function key(): Key;
}