<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\interfaces;

use apexl\Io\modules\user\entities\userEntity;

interface NewUserNotifierInterface
{
    public userEntity $user {
        set(userEntity $user);
    }
}
