<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\services;

use apexl\dto\Dto;
use apexl\Io\modules\user\interfaces\CanBeMasqueradedInterface;

final readonly class MasqueradeDto extends Dto
{

    /**
     * @param class-string<CanBeMasqueradedInterface> $masqueradedClass
     * @param int $id
     */
    public function __construct(
        public string $masqueradedClass,
        public int $id
    ) {}
}